#include "testApp.h"

bool hasStarted = false;

//--------------------------------------------------------------
void testApp::setup(){
	ofSetLogLevel(OF_LOG_VERBOSE);
	isDepthOn = false;
	isImageOn = false;
	configXMLPath = "OpenNIGrabber_config.xml";
	cameraWidth = 320;
	cameraHeight = 240;
	bNewPixels = false;
	isFrameNew = false;
	
	
	
	currentDepthPixels.allocate(cameraWidth, cameraHeight, OF_PIXELS_RGBA);
	depthTexture.allocate(cameraWidth, cameraHeight, GL_RGBA);
	
	XnStatus status = XN_STATUS_OK;
	xn::EnumerationErrors errors;
	string filePath = ofToDataPath(configXMLPath, true);
	
	status = context.InitFromXmlFile(filePath.c_str(), scriptNode, &errors);
	
	if(status!=XN_STATUS_OK)
	{
		
	}else {
		
		createGenerators();
		start();
	}

	ofLogVerbose() << "testApp::setup " << xnGetStatusString(status) << " : " << filePath;
	
}
void testApp::createGenerators()
{
	ofLogVerbose() << "testApp::createGenerators ";
	XnStatus status = XN_STATUS_OK;
	xn::NodeInfoList list;
	status = context.EnumerateExistingNodes(list);
	if (status == XN_STATUS_OK)
	{
		for (xn::NodeInfoList::Iterator it = list.Begin(); it != list.End(); ++it)
		{
			switch ((*it).GetDescription().Type)
			{
				case XN_NODE_TYPE_DEVICE:
					ofLogVerbose() << "Creating device";
					(*it).GetInstance(device);
					break;
				case XN_NODE_TYPE_DEPTH:
					ofLogVerbose() << "Creating depth generator";
					isDepthOn = true;
					isDepthRawOnOption = true;
					(*it).GetInstance(depthGenerator);
					break;
				case XN_NODE_TYPE_IMAGE:
					ofLogVerbose() << "Creating image generator";
					isImageOn = true;
					(*it).GetInstance(imageGenerator);
					break;
			}
		}
	}
	else 
	{
		ofLogVerbose() << "createGenerators::setup failed" << xnGetStatusString(status);
	}	
}

void testApp::start()
{
	ofLogVerbose() << "testApp::start";

	initConstants();
	allocateDepthBuffers();
	//allocateDepthRawBuffers();
	//allocateRGBBuffers();
	hasStarted = true;
}

void testApp::initConstants()
{
	ofLogVerbose() << "initConstants";
	
	int i = 0;
	
	primaryStream.name[i++] = "Any";
	primaryStream.name[i++] = xnProductionNodeTypeToString(XN_NODE_TYPE_DEPTH);
	primaryStream.name[i++] = xnProductionNodeTypeToString(XN_NODE_TYPE_IMAGE);
	primaryStream.name[i++] = xnProductionNodeTypeToString(XN_NODE_TYPE_IR);
	primaryStream.name[i++] = xnProductionNodeTypeToString(XN_NODE_TYPE_AUDIO);
	
	primaryStream.numValues = i;
	
	// Registration
	i = 0;
	
	registration.values[i++] = FALSE;
	registration.name[FALSE] = "Off";
	
	registration.values[i++] = TRUE;
	registration.name[TRUE] = "Depth -> Image";
	
	registration.numValues = i;
	
	// Resolutions
	i = 0;
	
	resolution.values[i++] = XN_RES_QVGA;
	resolution.name[XN_RES_QVGA] = xn::Resolution(XN_RES_QVGA).GetName();
	
	resolution.values[i++] = XN_RES_VGA;
	resolution.name[XN_RES_VGA] = xn::Resolution(XN_RES_VGA).GetName();
	
	resolution.values[i++] = XN_RES_SXGA;
	resolution.name[XN_RES_SXGA] = xn::Resolution(XN_RES_SXGA).GetName();
	
	resolution.values[i++] = XN_RES_UXGA;
	resolution.name[XN_RES_UXGA] = xn::Resolution(XN_RES_UXGA).GetName();
	
	resolution.numValues = i;
}
void testApp::allocateDepthBuffers()
{
	ofLogVerbose() << "allocateDepthBuffers" << isDepthOn;
	if(isDepthOn)
	{
		deviceMaxDepth	= depthGenerator.GetDeviceMaxDepth();
		
		//if(useTexture)
		//{
			
		//}
	}
}
//--------------------------------------------------------------
void testApp::update(){
	if (!hasStarted) 
	{
		ofLogVerbose() << "update NOT STARTED";
		return;
	}
	readFrame();
	if(bNewPixels)
	{
		if(isDepthOn)
		{
			depthTexture.loadData(currentDepthPixels);
		}
		/*if(isImageOn)
		{
			rgbTexture.loadData(*currentRGBPixels);
		}*/
		bNewPixels = false;
		isFrameNew = true;
	}else {
		isFrameNew = false;
	}
			
}

void testApp::generateDepthPixels()
{
	const XnDepthPixel* depthPixels = depthMetaData.Data();
	
	if (depthMetaData.FrameID() == 0) return;
	//ofLogVerbose() << "generateDepthPixels";
	
	
	// copy depth into texture map
	float max = 255;
	for (XnUInt16 y = depthMetaData.YOffset(); y < depthMetaData.YRes() + depthMetaData.YOffset(); y++) 
	{
		unsigned char * texture = currentDepthPixels.getPixels() + y * depthMetaData.XRes() * 4 + depthMetaData.XOffset() * 4;
		for (XnUInt16 x = 0; x < depthMetaData.XRes(); x++, depthPixels++, texture += 4)
		{
			XnUInt8 red = 0;
			XnUInt8 green = 0;
			XnUInt8 blue = 0;
			XnUInt8 alpha = 255;
			
			XnUInt16 col_index;
			
			XnUInt8 a = (XnUInt8)(((*depthPixels) / ( deviceMaxDepth / max)));
			red		= a;
			green	= a;
			blue	= a;
			
			texture[0] = red;
			texture[1] = green;
			texture[2] = blue;
			
			if (*depthPixels == 0)
			{
				texture[3] = 0;
			} else	
			{
				texture[3] = alpha;
			}
		}
	}
	
}
void testApp::readFrame()
{
	
	XnStatus status = XN_STATUS_OK;
	
	status = context.WaitAnyUpdateAll();
	//ofLogVerbose() << "context.WaitAnyUpdateAll " << xnGetStatusString(status);
	if (depthGenerator.IsValid())	depthGenerator.GetMetaData(depthMetaData);
	//if (imageGenerator.IsValid())	imageGenerator.GetMetaData(imageMetaData);	
	//if (irGenerator.IsValid())		irGenerator.GetMetaData(irMetaData);	
	if (isDepthOn)					generateDepthPixels();	
	//if (isImageOn)					generateImagePixels();
	

	/*if(isImageOn)
	{
		swap(backRGBPixels,currentRGBPixels);
	}*/
	bNewPixels = true;
}

//--------------------------------------------------------------
void testApp::draw(){
	if (!hasStarted) 
	{
		ofLogVerbose() << "draw NOT STARTED";
		return;
	}
	if(isDepthOn)
	{
		depthTexture.draw(20, 20, cameraWidth, cameraHeight);
	}
	ofDrawBitmapStringHighlight(ofToString(ofGetFrameRate()), ofGetWidth()/2, ofGetHeight()/2, ofColor::black, ofColor::yellow);
	

}
void testApp::exit(){
	ofLogVerbose() << "exit";
	if (hasStarted) 
	{
		//imageGenerator.StopGenerating();
		if (isDepthOn)
		{
			depthGenerator.StopGenerating();
		}
		context.Release();
		device.Release();	
	}
}	
//--------------------------------------------------------------
void testApp::keyPressed  (int key){

	
}

//--------------------------------------------------------------
void testApp::keyReleased(int key){

}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){

}

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button){


}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button){

}

//--------------------------------------------------------------
void testApp::mouseReleased(int x, int y, int button){

}


//--------------------------------------------------------------
void testApp::windowResized(int w, int h){

}

//--------------------------------------------------------------
void testApp::gotMessage(ofMessage msg){

}

//--------------------------------------------------------------
void testApp::dragEvent(ofDragInfo dragInfo){ 

}

